<?php

/** 
 * @file axioms.inc
 *
 * Defines the terms, and the terms that define the terms,
 * That make basic statements about what the statements mean :-B
 *
 * OK, sorry, axioms are the things that are so basic to the workings
 * of grammer that you take them for granted.
 * The term 'type' indicates the 'type' of a thing. Simple enough, 
 * but it has to be said before we can ask for all things of a given 'type'.
 *
 * Some basic vocabulary that forms the building blocks for later
 * definitions of statements. They are statements in themselves,
 * but really part of the core workings.
 *
 * Quirky thing to note is how sortOrder is used in the interface.
 * A term with sortOrder (weighting) less than zero does not show
 * up in most user lists, it's regarded as technical.
 * A term with sortOrder > 5 is so important it should even show up in summaries.
 * Thus you can tweak the interface available to editors without really
 * changing the properties of a predicate.
 * Setting the option 'List Advanced Ontologies' however will override this
 * filter and always show all.
 * 
 * Also, much of the settings here control how we co-operate with Drupal
 * internals. The definition of "what is a node" is expressed here in a 
 * standard form. Terms that are given a 'drupalEquivalent' characteristic
 * count as 'implicit' system properties, and require special rights to
 * access.
 *
 * @todo soon enough, all the statements made in this file should be re-parsed
 * into true RDF that I can bootstrap from. The struct array format used here
 * is was just used in the beginning, before I even had a data model.
 * The only question is - how much will requiring XML support on the server
 * limit the audience? (I'm not gonna parse by hand!)
 * Or should I redistribute the ARC thing as core?
 *
 * @ingroup docs
 */
 

/**
 * The namespaces we use to make the term definitions
 * @todo find a tidier way of looking after namespaces
 */
define ('RDF_NS',    'http://www.w3.org/1999/02/22-rdf-syntax-ns#');
define ('RDFS_NS',   'http://www.w3.org/2000/01/rdf-schema#');

define ('OWL_NS',    'http://www.w3.org/2002/07/owl#');
 
define ('DCTERMS_NS','http://purl.org/dc/terms/');
define ('DC_NS',     'http://purl.org/dc/elements/1.1/');


/**
 * ROR (Resources about Resources) will be used to encapsulate 
 * some of the extra drupal-specific defined properties
 * ror:sortOrder <==> Drupal:weight
 * We'll continue to use the term 'weight' internally for drupal-ness
 * but rename it and qualify it as ror:sortOrder for serialization
 */ 
define ('ROR_NS',    'http://rorweb.com/0.1/'); 


global $predicate_namespaces;
$predicate_namespaces = array(
  'rdf'     => RDF_NS,
  'rdfs'    => RDFS_NS,
  'owl'     => OWL_NS,
  'dcterms' => DCTERMS_NS,
  'dc'      => DC_NS,
  'ror'     => ROR_NS,
);

// Classes most often go on the recieving end of 'type' statements.
// so X is a typeOf 'Resource' etc.
 define('CLASS_PROPERTY',            'Property');
 define('CLASS_OBJECT_PROPERTY',     'objectProperty');
 define('CLASS_CLASS',               'Class');
 define('CLASS_RESOURCE',            'Resource');
 define('CLASS_LITERAL',             'Literal');
 define('CLASS_ONTOLOGY',            'Ontology');

 // Drupal - only 
 define('CLASS_NODE',                'node');
 define('CLASS_USER',                'user');
 define('CLASS_VOCABULARY',          'vocabulary');


// Predicates are actually known as 'properties' in RDF-speak,
// as some of them have literal objects.
 define('PREDICATE_TITLE',           'title');
 define('PREDICATE_LABEL',           'label');
 define('PREDICATE_IS_DEFINED_BY',   'isDefinedBy');
 define('PREDICATE_SEE_ALSO',        'seeAlso'); 
 define('PREDICATE_SAME_AS',         'sameAs'); 
 define('PREDICATE_COMMENT',         'comment');
 define('PREDICATE_TYPE',            'type');
 define('PREDICATE_SUB_PROPERTY_OF', 'subPropertyOf');
 define('PREDICATE_SORT_ORDER',      'sortOrder');
 define('PREDICATE_NAME',            'identifier');
 define('PREDICATE_VALUE',           'value');

 define('PREDICATE_DOMAIN',          'domain');
 define('PREDICATE_RANGE',           'range');

 define('PREDICATE_CAN_HAVE_PROPERTY','canHaveProperty'); // invented inverse of domain
 define('PREDICATE_CAN_USE_ONTOLOGY','canUseOntology'); // invented superset of canHaveProperty
 
 define('PREDICATE_INVERSE_OF',      'inverseOf');

 define('PREDICATE_IS_PART_OF',      'isPartOf');
 define('PREDICATE_DRUPAL_EQUIV',    'drupalEquivalent');


 define('PREDICATE_MODIFIED',        'modified' ); 
 define('PREDICATE_DESCRIPTION',     'description' ); 
 define('PREDICATE_CREATOR',         'creator' ); 
 define('PREDICATE_SORT_ORDER',      'sortOrder' ); 

// Ontologies are collections of terms, independant
// of what namespace the term was defined in 
// (although they do usually go together)
// predicate ontologies are analogous to term taxonomies.
// A term can be in as many ontologies as it likes.

 define('ONTOLOGY_RDF_AXIOMS', 'rdfAxioms'); 
 // these terms should be hidden by default - they are a bit techie

 define('ONTOLOGY_OWL', 'owlOntology'); 
 // these terms are extremely techie, they are used internally by the inference engine
 // By default, terms in this group will be hidden from the admin screen
 
 define('ONTOLOGY_RDF', 'rdfOntology'); 
 // these terms may be available for users

 define('ONTOLOGY_DRUPAL', 'drupalOntology');

 // an empty placeholder for users to put their own terms
 define('ONTOLOGY_LOCAL', 'localOntology');

// If a relationship is derived programatically by a lookup function
// (which means it can always be re-calculated later)
// set its 'depth' to this.
// This implies the storage should not remember this as it may mysteriously change.
 define('RELATIONSHIP_IS_CALCULATED',      -1);

// If a relationship is deduced logically
// (which usually means figuring it out again may be a lot of work)
// set its 'depth' to this.
// This implies the storage may cache it, but it can be discarded in favour of explicit statements.
 define('RELATIONSHIP_IS_DEDUCED',         2);

 
/**
 * Implimentation of hook_get_terms()
 *
 * Return a list of predicate term definitions
 * These terms are almost raw database inserts,  and
 * should get inserted into the database when the 
 * module is first installed.
 * 
 * This array defines the predicates, their terms and characteristics/behaviours.
 * @see ontologies hook_get_terms()
 */
function relationship_get_axioms() {

  // Following are basic concepts I need to just know before 
  // starting with RDF. Without the following terms, 
  // Knowlege cannot begin.
  // With them, everything else can be built
  
  $list = array(
    
    CLASS_PROPERTY => array (
      'name'       => CLASS_PROPERTY,
      'label'      => CLASS_PROPERTY,
      'type'       => CLASS_CLASS,
      'uri'        => RDF_NS . CLASS_PROPERTY,
      'isDefinedBy'=> RDF_NS,
      'description'=> 'The class of RDF properties..',
      'sortOrder'  => '-9',
      'isRequiredBy'=> 'this',
      'isPartOf'    => ONTOLOGY_RDF_AXIOMS,
    ),  
    PREDICATE_COMMENT => array (
      'name'       => PREDICATE_COMMENT,
      'label'      => PREDICATE_COMMENT,
      'type'       => CLASS_PROPERTY,
      'uri'        => RDFS_NS . PREDICATE_COMMENT,
      'isDefinedBy'=> RDFS_NS,
      'description'=> 'This is used to provide a human-readable description of a resource. ',
      'sortOrder'  => '-4',
      'isRequiredBy'=> 'this',
      'domain'     => CLASS_RESOURCE,
      'range'      => CLASS_LITERAL, 
      'isPartOf'   => ONTOLOGY_RDF,

    ),  
    PREDICATE_LABEL => array (
      'name'       => PREDICATE_LABEL,
      'label'      => PREDICATE_LABEL,
      'uri'        => RDFS_NS . PREDICATE_LABEL,
      'type'       => CLASS_PROPERTY,
      'isDefinedBy'=> RDFS_NS,
      'description'=> 'Provides a human-readable version of a resource name. The target of an rdfs:label arc must be a literal. The label may be a short title suitable for using in lists or navigation menus where the full document title would be too long.',
      'sortOrder'  => '0',
      'range'      => CLASS_LITERAL,
      'isRequiredBy'=> 'this',
      'isPartOf'   => ONTOLOGY_RDF,
    ),  
    PREDICATE_TYPE => array (
      'name'       => PREDICATE_TYPE,
      'label'      => PREDICATE_TYPE,
      'type'       => CLASS_PROPERTY,
      'uri'        => RDF_NS . PREDICATE_TYPE,
      //'http://purl.org/dc/elements/1.1/type',
      // Or is it ??!
      // http://www.w3.org/1999/02/22-rdf-syntax-ns#type
      'isDefinedBy'=> RDF_NS,
      'description'=> 'Identifies the Class of a resource. The target of an rdf:type arc must be an instance of rdfs:Class. ',
      'sortOrder'  => '0',
      'domain'     => array(CLASS_PROPERTY,'node'), // pretty much anything can have a 'type'
      'range'      => CLASS_CLASS,
      'isRequiredBy'=> 'this',
      'isPartOf'     => ONTOLOGY_RDF,
      PREDICATE_DRUPAL_EQUIV => 'type',
    ),  

    // The following definitions make life a little easier
    // They could also have been imported later.

    PREDICATE_IS_DEFINED_BY => array (
      'name'       => PREDICATE_IS_DEFINED_BY,
      'label'      => 'Is Defined By',
      'type'       => CLASS_PROPERTY,
      'uri'        => RDFS_NS . PREDICATE_IS_DEFINED_BY,
      'isDefinedBy'=> RDFS_NS,
      'description'=> 'Indicates a resource containing and defining the subject resource. The target of an rdfs:isDefinedBy arc must be a Resource. ',
      'sortOrder'  => '-9',
      'isPartOf'     => ONTOLOGY_RDF_AXIOMS,
    ),
   
    // This is used to serialize the 'sortOrder' parameter
    // Not totally identical with it, but close enough for most implimentations
    PREDICATE_SORT_ORDER => array (
      'name'       => PREDICATE_SORT_ORDER,
      'label'      => PREDICATE_SORT_ORDER,
      'type'       => CLASS_PROPERTY,
      'uri'        => ROR_NS . PREDICATE_SORT_ORDER,
      'isDefinedBy'=> ROR_NS,
      'description'=> '
          Order position of resource (e.g. 1, 2, 3, etc). 
          NOTE this value is being used to represent the Drupal weight property. 
      ',
      'sortOrder'  => '-9',
      'range'      => CLASS_LITERAL, 
      'isPartOf'     => ONTOLOGY_RDF_AXIOMS,
    ),  

   // Would you believe neither W3C nor DC defines the meaning of 'name' ??
   // http://dublincore.org/documents/dcmi-terms/#H2 tells me that
   // Name:  The unique token assigned to the term. 
   // BUT then fails to actually serialize it in the associated RDF.
   // I see mention of rdf:ID - but no definition in the rdf schema.
   // I have to deduce it as being the last part of the URI.
   // The closest I can find to do this trick is 'identifier'
   // http://purl.org/dc/elements/1.1/identifier
   
    PREDICATE_NAME => array (
      'name'       => PREDICATE_NAME,
      'label'      => PREDICATE_NAME,
      // this is causing troubles -todo
      //'type'       => 'http://dublincore.org/usage/documents/principles/#element',
      'uri'        => DC_NS . PREDICATE_NAME,
      'isDefinedBy'=> DC_NS,
      'description'=> '
          An unambiguous reference to the resource within a given context. 
          This is a LOCAL name, not a full URI.
      ',
      'sortOrder'  => '0',
      'isPartOf'     => ONTOLOGY_RDF_AXIOMS,
      // PREDICATE_DRUPAL_EQUIV => 'nid', //?
    ),  

    //  definitions of terms needed to know and define what an 'ontology' group is.
    CLASS_ONTOLOGY => array (
      'name'       => CLASS_ONTOLOGY,
      'uri'        => OWL_NS . CLASS_ONTOLOGY,
      'label'      => 'an Ontology',
      'description'=> 'Represents an ontology object class',
      'type'       => CLASS_CLASS,
      'isPartOf'   => ONTOLOGY_RDF_AXIOMS,
    ),

    // isPartOf is problematic - does deleting the container delete the members?
    // Are the parts logically subservient to the whole, or is it just
    // a grouping thing? Maybe I should use isMemberOf instead, where disolving the
    // group obviously does not affect the members.
    // For now, I'll use isPartOf as being non-strict, 
    // things can be part of more than one other thing.
    PREDICATE_IS_PART_OF => array (
      'name'       => PREDICATE_IS_PART_OF,
      'uri'        => DCTERMS_NS . PREDICATE_IS_PART_OF,
      'label'      => 'Is Part Of',
      'description'=> 'The described resource is a physical or logical part of the referenced resource.',
      'type'       => CLASS_PROPERTY,
      'domain'     => CLASS_PROPERTY,
      'range'      => CLASS_ONTOLOGY,
      'inverseOf'  => DCTERMS_NS .'hasPart', 
      'isPartOf'   => ONTOLOGY_RDF,
 
    ),
 

    ONTOLOGY_RDF_AXIOMS => array (
      'name'       => ONTOLOGY_RDF_AXIOMS,
      'type'       => CLASS_ONTOLOGY,
      'sortOrder'  => '-5',
      'description'=> 'A group of reasonably technical, internal terms. Building blocks of the ontologies themselves. Hidden by default.',
      'isPartOf'   => ONTOLOGY_RDF_AXIOMS,
    ),  
    ONTOLOGY_RDF   => array (
      'name'       => ONTOLOGY_RDF,
      'type'       => CLASS_ONTOLOGY,
      'sortOrder'  => '-5',
      'description'=> '
          A group of reasonably useful, reasonably generic terms, 
          suitable for re-use in user ontologies.
      ',
      'isPartOf'   => ONTOLOGY_RDF,
    ),  
    

  ); // list

  return $list;
}

function relationship_get_more_axioms() {

  // Useful things to know when working with RDF, but not vital
  // Best to go import them
  return array(
    PREDICATE_INVERSE_OF => array(
    'name'         => PREDICATE_INVERSE_OF,
    'label'        => 'is the inverse (reciprocal relationship) of',
    'uri'          => OWL_NS. PREDICATE_INVERSE_OF,
    'type'         => CLASS_PROPERTY,
    'description'  => '
        If A bears a relation to B,  then B implicitly has the reciprocal
        relation to B,  
        eg "IsFollowedBy" has the inverse relation "IsPrecededBy"
    ',
    'domain'       => array(CLASS_OBJECT_PROPERTY,CLASS_PROPERTY),
    'range'        => array(CLASS_OBJECT_PROPERTY,CLASS_PROPERTY),
    'isPartOf'     => ONTOLOGY_OWL,
    'sortOrder'    => 5,
  ),


    PREDICATE_SEE_ALSO => array (
      'name'       => PREDICATE_SEE_ALSO,
      'label'      => 'See Also',
      'type'       => CLASS_PROPERTY,
      'uri'        => 'http://www.w3.org/2000/01/rdf-schema#seeAlso',
      'isDefinedBy'=> RDFS_NS,
      'comment' => '
          Indicates a resource that provides information about the subject resource.
          The target of an rdfs:seeAlso arc must be a Resource. ',
      'sortOrder'  => '0',
      'isPartOf'    => ONTOLOGY_RDF,
   ),  

    PREDICATE_SUB_PROPERTY_OF => array (
      'name'       => PREDICATE_SUB_PROPERTY_OF,
      'label'      => 'Is A Sub Property Of',
      'type'       => CLASS_PROPERTY,
      'uri'        => 'http://www.w3.org/2000/01/rdf-schema#subPropertyOf',
      'isDefinedBy'=> DCTERMS_NS,
      'comment' => 'Indicates that one predicate is a refinement of another',
      'sortOrder'  => '-9',
      'isPartOf'    => ONTOLOGY_RDF_AXIOMS,
      'domain'     => CLASS_PROPERTY,
      'range'      => CLASS_PROPERTY,
   ),  

    PREDICATE_SAME_AS => array (
      'name'       => PREDICATE_SAME_AS,
      'label'      => 'Is The Same As',
      'type'       => CLASS_PROPERTY,
      'uri'        => OWL_NS . PREDICATE_SAME_AS,
      'isDefinedBy'=> OWL_NS,
      'comment' => '
          Indicates that two identifiers refer to exactly the same thing.
          All properties between them should be shared.
      ',
      'inverseOf'  => PREDICATE_SAME_AS,
      'sortOrder'  => '2',
      'isPartOf'    => ONTOLOGY_OWL,
   ),  

    // This is used in association with type.
    CLASS_CLASS => array (
      'name'       => CLASS_CLASS,
      'label'      => CLASS_CLASS,
      'type'       => CLASS_CLASS,
      'uri'        => 'http://www.w3.org/2000/01/rdf-schema#Class',
      'isDefinedBy'=> RDFS_NS,
      'comment' => 'The concept of Class ',
      'sortOrder'  => '-9',
      'isPartOf'    => ONTOLOGY_RDF_AXIOMS,
   ),  

    CLASS_RESOURCE => array (
      'name'       => CLASS_RESOURCE,
      'label'      => 'The Generic super-class',
      'type'       => CLASS_CLASS,
      'uri'        => RDF_NS . CLASS_RESOURCE,
      'isDefinedBy'=> RDFS_NS,
      'comment'    => 'The Generic super-class',
      'sortOrder'  => '-9',
      'isPartOf'   => ONTOLOGY_RDF_AXIOMS,
   ),  

    // Do I need this?
    PREDICATE_VALUE => array (
      'name'       => PREDICATE_VALUE,
      'label'      => PREDICATE_VALUE,
      'uri'        => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#value',
      'isDefinedBy'=> RDF_NS,
      'description'=> 'Identifies the principal value (usually a string) of a property.',
      'sortOrder'  => '-9',
      'isPartOf'    => ONTOLOGY_RDF_AXIOMS,
     ),  
    CLASS_LITERAL  => array (
      'name'       => CLASS_LITERAL,
      'label'      => CLASS_LITERAL,
      'type'       => CLASS_CLASS,
      'uri'        => 'http://www.w3.org/2000/01/rdf-schema#Literal',
      'isDefinedBy'=> RDFS_NS,
      'description'=> 'The class of literal values, eg. textual strings and integers.',
      'sortOrder'  => '-9',
      'isPartOf'    => ONTOLOGY_RDF_AXIOMS,
     ),  

    PREDICATE_DOMAIN => array (
      'name'       => PREDICATE_DOMAIN,
      'label'      => 'Applies To',
      'uri'        => RDFS_NS . PREDICATE_DOMAIN,
      'description'=> '
          The types of subjects this term is a valid property for. 
          What this property can be applied to.
          A book may have an associated author, therefore the "author" 
          relationship has the domain of "book" ... and others.
          
          A predicate may have more than one domain.
          A predicate with NO domain can apply to any resource.
          
          When creating a predicate that is supposed to apply to a node,
          the predicate should be given a domain of "node". If the 
          predicate should only apply to a certain node type, the domain
          is the name of that node type, 
          eg location:domain:event - an Event node type may have a location property.
      ',
      'sortOrder'  => '-1',
      'isPartOf'   => ONTOLOGY_RDF_AXIOMS,
      'domain'     => array(CLASS_CLASS, CLASS_PROPERTY), // class Property is of course a subClassOf class Class and should inheirit the relationship, but that's complex
      'range'      => CLASS_CLASS,
      'inverseOf'  => PREDICATE_CAN_HAVE_PROPERTY,
     ),  

    PREDICATE_CAN_HAVE_PROPERTY => array (
      'name'       => PREDICATE_CAN_HAVE_PROPERTY,
      'label'      => 'Can Have Property',
      'description'=> '
          A predicate this class may have set.
          A property resources of this class may exhibit.
      ',
      'isPartOf'   => ONTOLOGY_RDF_AXIOMS,
      'type'       => CLASS_PROPERTY,
      'domain'     => CLASS_CLASS,
      'range'      => CLASS_PROPERTY,
      'inverseOf'  => PREDICATE_DOMAIN,
     ),  

    PREDICATE_CAN_USE_ONTOLOGY => array (
      'name'       => PREDICATE_CAN_USE_ONTOLOGY,
      'label'      => 'Can Use Ontology',
      'description'=> '
          Indicates that terms from the referenced ontology set can apply to resources of this type.
      ',
      'isPartOf'   => ONTOLOGY_RDF_AXIOMS,
      'type'       => CLASS_PROPERTY,
      'domain'     => CLASS_CLASS,
      'range'      => CLASS_ONTOLOGY,
     ),  



    PREDICATE_RANGE => array (
      'name'       => PREDICATE_RANGE,
      'label'      => 'Can Have Value Of Type',
      'uri'        => RDFS_NS . PREDICATE_RANGE,
      'description'=> '
            The types of objects that are expected to be at the other end of 
            the relationship. The "author" relationship is expected to point
            to a type "person", so the "range" of "author" is "person" 
            (which probably means a personal user page) 
            ... and probably not much else. Only people author stuff... 
            The range may also be set to "Literal" or "Integer" meaning only
            strings or numbers are valid values. 
            If the range of a predicate is not set, the default range is 
            "resource" which just means some other URI. If it IS set, the
            allowable values are limited to the named class(s). Multiple 
            ranges are allowed.
      ',
      'sortOrder'  => '-1',
      'isPartOf'   => ONTOLOGY_RDF_AXIOMS,
      'domain'     => CLASS_PROPERTY,
      'range'      => CLASS_CLASS,
     ), 
    ONTOLOGY_OWL => array (
      'name'       => ONTOLOGY_OWL,
      'type'       => CLASS_ONTOLOGY,
      'isPartOf'    => ONTOLOGY_OWL,
      'sortOrder'  => '-5',
      'description'=> '
          Terms used by OWL - the Web Ontology Language. 
          Technical predicates used only for applying characteristics and
          properties to predicates themselves.
      ',
    ),  
     
    // These are to support the Drupal Node Data/Metadata in international (DC) syntax.
    //
    // Giving a property a drupalEquiv value is a hint to extracting 
    // Drupal object properties into common form.
    // EG, I convert the Drupal "$node->changed" into "DC:modified" before saving it.
    // 
    // If a property has multiple drupalEquiv values,
    // or multiple properties claim the same equiv it may get
    // confusing. Weighting will be used in case of a tie.
    // Custom nodes can of course extend into this!
    //
    // Even if the mapping doesn't change the word used (eg $node->type = RDFS:type)
    // the existance of the mapping indicates that the property is worth abstracting
    // from a complex node object.
    //
    // I'll move these and relationship_get_property_map() into a module of their own soon.
    //
    ONTOLOGY_DRUPAL => array (
      'name'       => ONTOLOGY_DRUPAL,
      'type'       => CLASS_ONTOLOGY,
      'isPartOf'    => ONTOLOGY_DRUPAL,
      'description'=> '
          Terms used by Drupal to represent information usually contained in nodes.
          Internal definitions - like user module-defined node types - are grouped here.
      ',
    ),  

    CLASS_NODE => array (
      'name'       => CLASS_NODE,
      'label'      => CLASS_NODE,
      'type'       => CLASS_CLASS,
      'comment' => 'The class of Drupal Nodes. Content types (like page, story and flexinode) are subclasses of node.',
      'isPartOf'   => ONTOLOGY_DRUPAL,
   ),  

    CLASS_VOCABULARY => array (
      'name'       => CLASS_VOCABULARY,
      'label'      => CLASS_VOCABULARY,
      'type'       => CLASS_CLASS,
      'comment' => 'The class of Drupal Vocabularies. User-defined vocabularies are implicitly a subclass of this',
      'isPartOf'   => ONTOLOGY_DRUPAL,
   ),  

    PREDICATE_DRUPAL_EQUIV => array (
      'name'       => PREDICATE_DRUPAL_EQUIV,
      'label'      => 'is known in Drupal as',
      'type'       => CLASS_PROPERTY,
      'comment'    => 'This term is equivalent to the internal Drupal node property.',
      'sortOrder'  => '2',
      'domain'     => CLASS_PROPERTY,
      'range'      => CLASS_LITERAL,
      'isPartOf'   => ONTOLOGY_DRUPAL,
    ),    
    
    PREDICATE_MODIFIED => array (
      'name'       => PREDICATE_MODIFIED,
      'label'      => 'was last modified on',
      'type'       => CLASS_PROPERTY,
      'uri'        => DCTERMS_NS.PREDICATE_MODIFIED,
      'isDefinedBy'=> DCTERMS_NS,
      'comment'    => 'The date that this resource was last edited.',
      'sortOrder'  => '2',
      'domain'     => CLASS_NODE,
      'range'      => CLASS_LITERAL, // really should be http://dublincore.org/documents/dcmi-terms/ date format, but that can be done later
      'isPartOf'   => ONTOLOGY_DRUPAL,
      PREDICATE_DRUPAL_EQUIV => 'changed',
    ),  
    PREDICATE_TITLE => array (
      'name'       => PREDICATE_TITLE,
      'label'      => 'is titled',
      'type'       => CLASS_PROPERTY,
      'uri'        => DCTERMS_NS.PREDICATE_TITLE,
      'isDefinedBy'=> DCTERMS_NS,
      'comment'    => 'The full title or name of this item.',
      'sortOrder'  => '1',
      'domain'     => CLASS_RESOURCE,
      'range'      => CLASS_LITERAL,
      'isPartOf'   => ONTOLOGY_DRUPAL,
      PREDICATE_DRUPAL_EQUIV => 'title',
    ),  
    PREDICATE_DESCRIPTION => array (
      'name'       => PREDICATE_DESCRIPTION,
      'label'      => 'Description',
      'type'       => CLASS_PROPERTY,
      'uri'        => DC_NS.PREDICATE_DESCRIPTION,
      'isDefinedBy'=> DC_NS,
      'comment'    => 'An account of the content of the resource.',
      'sortOrder'  => '2',
      'domain'     => array(CLASS_NODE,CLASS_RESOURCE),
      'range'      => CLASS_LITERAL,
      'isPartOf'   => ONTOLOGY_DRUPAL,
      PREDICATE_DRUPAL_EQUIV => 'teaser',
    ),  
    PREDICATE_CREATOR => array (
      'name'       => PREDICATE_CREATOR,
      'label'      => 'Was Created By',
      'type'       => CLASS_PROPERTY,
      'uri'        => DC_NS.PREDICATE_CREATOR,
      'isDefinedBy'=> DC_NS,
      'comment'    => '
          An entity primarily responsible for making the content of the resource.;
          Examples of Creator include a person, an organization, or a service. 
          Typically, the name of a Creator should be used to indicate the entity. 
      ',
      'sortOrder'  => '2',
      'domain'     => array(CLASS_NODE,CLASS_RESOURCE),
      'range'      => CLASS_USER,
      'isPartOf'   => ONTOLOGY_DRUPAL,
      PREDICATE_DRUPAL_EQUIV => 'teaser',
    ),  
    PREDICATE_SORT_ORDER => array (
      'name'       => PREDICATE_SORT_ORDER,
      'label'      => 'Is Weighted',
      'type'       => CLASS_PROPERTY,
      'isPartOf'   => ONTOLOGY_DRUPAL,
      PREDICATE_DRUPAL_EQUIV => 'weight',
    ),
      
    ONTOLOGY_LOCAL => array (
      'name'       => ONTOLOGY_LOCAL,
      'type'       => CLASS_ONTOLOGY,
      'isPartOf'    => ONTOLOGY_LOCAL,
      'description'=> '
          Terms described only on this installation.
      ',
    ),  

  );
}







/**
 * Return the property map for a Drupal-defined content type.
 *
 * Although I can represent core Drupal data structures (node, user etc)
 * using formal terminology, sometimes the properties need to be renamed.
 * This function returns a mapping array for various existing data types.
 *
 * Properties NOT in this list will be dealt with as normal arbitrary listed
 * predicates, which get serialized as-is, but don't get any special 
 * treatment.
 * Properties that ARE listed here are expected 
 * to be serialized individually by the Drupal form interface.
 *
 * The arrays are also used as for mapping Drupal-specific terminology to 
 * other terms from existing namespaces
 * otherns:term => drupal:term 
 * predicate => property
 *
 * eg, 'name' isn't a real RDF term, we'll save it
 * as 'identifier' but let Drupal keep calling it the 'name' property.
 *
 * Use this function call to see if a statement has a similar Drupal property
 * that should get filled in.
 * (Reading from triple-store into node def)
 *
 * Use array_flip to find the inverse, and make sure Drupal
 * 'parameters' get saved out of the object as 'statements'.
 * (When serializing a node into the triple store)
 * Note that this may cause some data compression when converting to cannonical
 * format. Multiple Drupal labels can be associated with the same DC label,
 * ($node->flexinode-3, $node->myname, $node->title) == DC:title
 * but not vice versa.
 *
 * This method can be used to 'cast' an existing node type (with arbitrary
 * fields) into a set of statements. It supports arrays also, so if you map
 *
 *      'dc:subject'  => 'taxonomy'
 *
 * then 
 *      $node->taxonomy = ['sex','drugs','rock&roll']
 *
 * will be mapped to the corresponding
 *
 *      node : dc:subject : 'sex'
 *      node : dc:subject : 'drugs'
 *      node : dc:subject : 'rock&roll'
 *
 * statements and serialized.
 *
 * @todo allow custom modules to populate this, they'll need it. 
 * 
 * The $drupal_type parameter is now deprecated, the DRUPAL_EQUIV 
 * characteristic now configures this.
 *
 * @param $flush if set, the cache is flushed and a new array generated
 */
 
function relationship_get_property_map($drupal_type='',$flush=FALSE) {
  static $map;
  if($map){return $map;}
  
  $property_map_cache_key = 'relationship_property_map';
  if( (!$flush) && ($cached = cache_get($property_map_cache_key) ) ){
    $map = unserialize($cached->data);
  } else {

    static $loop;
    // Due to the self-referential nature of this, we may get recursion panic
    // on bootstrapping. If so, just return the hardcoded basics.
    // 'name' is dc:identifier and 'description' is rdf:comment 
    $built_in_mappings = array(PREDICATE_NAME => 'name', PREDICATE_COMMENT => 'description' );
    if($loop){
      debug("Recursion panic, returning hardcoded property map",2);
      // debug_pre( debug_backtrace ( ) );
      return $built_in_mappings;
    } // recursion panic
    $loop = TRUE;
    
    $statements = relationship_query(NULL,PREDICATE_DRUPAL_EQUIV,NULL);
    // other_ns:term => drupal:property
    $map = relationship_get_assoc_column($statements,'subject','object',TRUE);
    $map = array_merge($built_in_mappings,$map);
    cache_set($property_map_cache_key,serialize($map)) ;
  }
  return $map;
}

